@@ -1,71 +1,8 @@
 #' The DataSpaceConnection class
 #'
-#' @return an instance of \code{DataSpaceConnection}
-#'
 #' @section Constructor:
 #' \code{\link{connectDS}}
 #'
-#' @section Fields:
-#' \describe{
-#'   \item{\code{config}}{
-#'     A list. Stores configuration of the connection object such as
-#'     URL, path and username.
-#'   }
-#'   \item{\code{availableStudies}}{
-#'     A data.table. The table of available studies.
-#'   }
-#'   \item{\code{availableGroups}}{
-#'     A data.table. The table of available groups.
-#'   }
-#'   \item{\code{mabGrid}}{
-#'     A data.table. The filtered mAb grid.
-#'   }
-#'   \item{\code{mabGridSummary}}{
-#'     A data.table. The filtered grid with updated \code{n_} columns and
-#'     \code{geometric_mean_curve_ic50}.
-#'   }
-#' }
-#'
-#' @section Methods:
-#' \describe{
-#'   \item{\code{initialize(login = NULL, password = NULL, verbose = FALSE,
-#'   onStaging = FALSE)}}{
-#'     Initialize a \code{DataSpaceConnection} object.
-#'     See \code{\link{connectDS}}.
-#'   }
-#'   \item{\code{print()}}{
-#'     Print the \code{DataSpaceConnection} object.
-#'   }
-#'   \item{\code{getStudy(study, groupId = NULL)}}{
-#'     Create a \code{\link{DataSpaceStudy}} object.
-#'
-#'     \code{study}: A character. Name of the study to retrieve.
-#'   }
-#'   \item{\code{getGroup(groupId)}}{
-#'     Create a \code{\link{DataSpaceStudy}} object.
-#'
-#'     \code{groupId}: An integer. ID of the group to retrieve.
-#'   }
-#'   \item{\code{refresh()}}{
-#'     Refresh the connection object to update available studies and groups.
-#'   }
-#'   \item{\code{filterMabGrid(using, value)}}{
-#'     Filter rows in the mAb grid by specifying the values to keep in the
-#'     columns found in the \code{mabGrid} field. It takes the column and the
-#'     values and filters the underlying tables.
-#'
-#'     \code{using}: A character. Name of the column to filter.
-#'
-#'     \code{value}: A character vector. Values to keep in the mAb grid.
-#'   }
-#'   \item{\code{getMab()}}{
-#'     Create a \code{\link{DataSpaceMab}} object.
-#'   }
-#'   \item{\code{resetMabGrid()}}{
-#'     Reset the mAb grid to the unfiltered state.
-#'   }
-#' }
-#'
 #' @seealso \code{\link{connectDS}} \code{\link{DataSpaceR-package}}
 #'
 #' @examples
@@ -89,9 +26,6 @@
 #' con$refresh()
 #' }
 #'
-#' @docType class
-#' @format NULL
-#'
 #' @importFrom jsonlite fromJSON
 #' @importFrom curl has_internet nslookup
 #' @importFrom Rlabkey labkey.selectRows
@@ -99,6 +33,20 @@
 DataSpaceConnection <- R6Class(
   classname = "DataSpaceConnection",
   public = list(
+
+    #' @description
+    #' Initialize a \code{DataSpaceConnection} object.
+    #' See \code{\link{connectDS}}.
+    #' @param login A character. Optional argument. If there is no netrc file a
+    #' temporary one can be written by passing login and password of an active
+    #' DataSpace account.
+    #' @param password A character. Optional. The password for the selected
+    #' login.
+    #' @param verbose A logical. Whether to print the extra details for
+    #' troubleshooting.
+    #' @param onStaging A logical. Whether to connect to the staging server
+    #' instead of the production server.
+    #' @return A new `DataSpaceConnection` object.
     initialize = function(login = NULL,
                           password = NULL,
                           verbose = FALSE,
@@ -155,6 +103,9 @@ DataSpaceConnection <- R6Class(
 
       NULL
     },
+
+    #' @description
+    #' Print the \code{DataSpaceConnection} object.
     print = function() {
       cat("<DataSpaceConnection>")
       cat("\n  URL:", private$.config$labkeyUrlBase)
@@ -164,8 +115,14 @@ DataSpaceConnection <- R6Class(
       cat("\n    -", private$.stats$subjects, "subjects")
       cat("\n    -", private$.stats$datacount, "data points")
       cat("\n  Available groups:", nrow(private$.availableGroups))
+      cat("\n  Available publications:", nrow(private$.availablePublications))
+      cat("\n    -", sum(private$.availablePublications$publication_data_available), "publications with data")
       cat("\n")
     },
+
+    #' @description
+    #' Create a \code{\link{DataSpaceStudy}} object.
+    #' @param study A character. Name of the study to retrieve.
     getStudy = function(study) {
       if (study != "") {
         studyInfo <- as.list(
@@ -177,6 +134,10 @@ DataSpaceConnection <- R6Class(
 
       DataSpaceStudy$new(study, private$.config, NULL, studyInfo)
     },
+
+    #' @description
+    #' Create a \code{\link{DataSpaceStudy}} object.
+    #' @param groupId An integer. ID of the group to retrieve.
     getGroup = function(groupId) {
       assert_that(
         is.number(groupId),
@@ -192,6 +153,13 @@ DataSpaceConnection <- R6Class(
 
       DataSpaceStudy$new("", private$.config, group, NULL)
     },
+
+    #' @description
+    #' Filter rows in the mAb grid by specifying the values to keep in the
+    #' columns found in the \code{mabGrid} field. It takes the column and the
+    #' values and filters the underlying tables.
+    #' @param using A character. Name of the column to filter.
+    #' @param value A character vector. Values to keep in the mAb grid.
     filterMabGrid = function(using, value) {
       assertColumn(using, self)
 
@@ -239,6 +207,9 @@ DataSpaceConnection <- R6Class(
 
       invisible(self)
     },
+
+    #' @description
+    #' Reset the mAb grid to the unfiltered state.
     resetMabGrid = function() {
       private$.mabGridBase <- private$.cache$mabGridBase
       private$.mabMetaGridBase <- private$.cache$mabMetaGridBase
@@ -246,9 +217,84 @@ DataSpaceConnection <- R6Class(
 
       invisible(self)
     },
+
+    #' @description
+    #' Create a \code{\link{DataSpaceMab}} object.
     getMab = function() {
       DataSpaceMab$new(self$mabGridSummary$mab_mixture, private$.mabFilters, private$.config)
     },
+
+    #' @description
+    #' Download publication data for a chosen publication.
+    #' @param publicationID A character/integer. ID for the publication to
+    #' download data for.
+    #' @param outputDir A character. Path to directory to download publication
+    #' data.
+    #' @param unzip A logical. If TRUE, unzip publication data to outputDir.
+    #' @param verbose A logical. Default TRUE.
+    downloadPublicationData = function(publicationID,
+                                       outputDir = getwd(),
+                                       unzip = TRUE,
+                                       verbose = TRUE) {
+      assert_that(
+        dir.exists(outputDir),
+        msg = paste0(outputDir, " is not a directory")
+      )
+      assert_that(
+        publicationID %in% private$.availablePublications$publication_id,
+        msg = paste0(publicationID, " is not a valid publicationID")
+      )
+      assert_that(
+        private$.availablePublications[publication_id == publicationID]$publication_data_available,
+        msg = paste0("No publication data available for publication ", publicationID)
+      )
+      assert_that(is.logical(verbose))
+
+      remotePath <- private$.availablePublications[publication_id == publicationID]$remotePath
+      fileName <- basename(remotePath)
+      localZipPath <- file.path(outputDir, fileName)
+      fullOutputDir <- file.path(outputDir, gsub(".zip", "", fileName))
+      if (verbose) message("downloading ", fileName, " to ", outputDir, " ...")
+
+      # Use getStudyDocumentUrl.view to download
+      getStudyDocumentUrl <- paste0(
+        private$.config$labkeyUrlBase,
+        "/cds/CAVD/getStudyDocument.view?",
+        "&documentId=", private$.availablePublications[publication_id == publicationID]$document_id,
+        "&filename=", gsub("/", "%2F", remotePath),
+        "&publicAccess=true"
+      )
+
+      # Use labkey.webdav.getByUrl which includes filesystem and permissions checks
+      ret <- Rlabkey:::labkey.webdav.getByUrl(getStudyDocumentUrl, localFilePath = localZipPath, overwrite = TRUE)
+      if (!is.null(ret) && !is.na(ret) && ret == FALSE) {
+        success <- FALSE
+        stop("failed to download ", fileName)
+      } else {
+        success <- file.exists(localZipPath)
+        if (!success) stop("failed to download to", localZipPath)
+      }
+
+      if (verbose) {
+        message("File Contents: ")
+        fileContents <- capture.output(unzip(localZipPath, list = TRUE))
+        message(paste0(fileContents, collapse = "\n"))
+      }
+
+      if (unzip) {
+        if (verbose) message("unzipping ", fileName, " to ", fullOutputDir)
+        unzippedFiles <- unzip(localZipPath, exdir = fullOutputDir)
+        unlink(localZipPath)
+        # Return vector of file paths
+        return(invisible(unzippedFiles))
+      }
+
+      # Return path of zip file
+      invisible(localZipPath)
+    },
+
+    #' @description
+    #' Refresh the connection object to update available studies and groups.
     refresh = function() {
       tries <- c(
         class(try(
@@ -266,6 +312,14 @@ DataSpaceConnection <- R6Class(
         class(try(
           private$.getMabGrid(),
           silent = !private$.config$verbose
+        )),
+        class(try(
+          private$.getVirusMetadata(),
+          silent = !private$.config$verbose
+        )),
+        class(try(
+          private$.getAvailablePublications(),
+          silent = !private$.config$verbose
         ))
       )
 
@@ -273,15 +327,31 @@ DataSpaceConnection <- R6Class(
     }
   ),
   active = list(
+
+    #' @field config A list. Stores configuration of the connection object such as
+    #' URL, path and username.
     config = function() {
       private$.config
     },
+
+    #' @field availableStudies A data.table. The table of available studies.
     availableStudies = function() {
       private$.availableStudies
     },
+
+    #' @field availableGroups A data.table. The table of available groups.
     availableGroups = function() {
       private$.availableGroups
     },
+
+    #' @field availablePublications A data.table. The table of available
+    #' publications.
+    availablePublications = function() {
+      private$.availablePublications[, !c("document_id", "remotePath")]
+    },
+
+    #' @field mabGridSummary A data.table. The filtered grid with updated
+    #' \code{n_} columns and \code{geometric_mean_curve_ic50}.
     mabGridSummary = function() {
       mabGridBase <- copy(private$.mabGridBase)
       mabMetaGridBase <- copy(private$.mabMetaGridBase)
@@ -325,6 +395,8 @@ DataSpaceConnection <- R6Class(
 
       mabGrid[, .(mab_mixture, donor_species, isotype, hxb2_location, n_viruses, n_clades, n_tiers, geometric_mean_curve_ic50, n_studies)]
     },
+
+    #' @field mabGrid A data.table. The filtered mAb grid.
     mabGrid = function() {
       mabGridBase <- copy(private$.mabGridBase)
       mabMetaGridBase <- copy(private$.mabMetaGridBase)
@@ -357,7 +429,6 @@ DataSpaceConnection <- R6Class(
         ]
       )
 
-
       setkey(mabGridBase, mab_mix_id)
       setkey(mabMetaGridBase, mab_mix_id)
 
@@ -369,6 +440,12 @@ DataSpaceConnection <- R6Class(
       )
 
       mabGrid[, .(mab_mixture, donor_species, isotype, hxb2_location, virus, clade, tier, curve_ic50, study)]
+    },
+
+    #' @field virusMetadata A data.table. Metadata about all viruses in the
+    #' DataSpace.
+    virusMetadata = function() {
+      private$.virusMetadata
     }
   ),
   private = list(
@@ -379,6 +456,8 @@ DataSpaceConnection <- R6Class(
     .mabGridBase = data.table(),
     .mabMetaGridBase = data.table(),
     .mabFilters = list(),
+    .virusMetadata = data.table(),
+    .availablePublications = data.table(),
     .cache = list(),
 
     .getAvailableStudies = function() {
@@ -403,6 +482,7 @@ DataSpaceConnection <- R6Class(
 
       private$.availableStudies <- availableStudies
     },
+
     .getStats = function() {
       stats <- labkey.selectRows(
         baseUrl = private$.config$labkeyUrlBase,
@@ -417,6 +497,7 @@ DataSpaceConnection <- R6Class(
 
       private$.stats <- stats
     },
+
     .getAvailableGroups = function() {
       participantGroupApi <- paste0(
         private$.config$labkeyUrlBase,
@@ -462,6 +543,7 @@ DataSpaceConnection <- R6Class(
 
       private$.availableGroups <- availableGroups
     },
+
     .getMabGrid = function() {
       mabGridBase <- labkey.selectRows(
         baseUrl = private$.config$labkeyUrlBase,
@@ -490,6 +572,78 @@ DataSpaceConnection <- R6Class(
       private$.cache$mabMetaGridBase <- copy(mabMetaGridBase)
 
       invisible(NULL)
+    },
+
+    .getVirusMetadata = function() {
+      colSelect <- c(
+        "assay_identifier", "virus", "virus_type", "neutralization_tier", "clade",
+        "antigen_control", "virus_full_name", "virus_name_other", "virus_species",
+        "virus_host_cell", "virus_backbone", "panel_names"
+      )
+
+      virusMetadata <- labkey.selectRows(
+        baseUrl = private$.config$labkeyUrlBase,
+        folderPath = "/CAVD/",
+        schemaName = "CDS",
+        queryName = "nabAntigenWithPanelMeta",
+        colSelect = colSelect,
+        colNameOpt = "fieldname",
+        method = "GET"
+      )
+      setDT(virusMetadata)
+      setkey(virusMetadata, virus)
+
+      private$.virusMetadata <- virusMetadata
+    },
+
+    .getAvailablePublications = function() {
+      sqlQuery <-
+        "
+SELECT publication.id as publication_id, author_first first_author, title, journal_short journal, date publication_date,
+link, pmid as pubmed_id, related_studies, studies_with_data, filename IS NOT NULL as publication_data_available, document_id,
+filename as remote_path
+FROM publication
+LEFT OUTER JOIN
+  (
+    SELECT GROUP_CONCAT(DISTINCT prot, ', ') AS related_studies, publication_id
+    FROM studypublication
+    GROUP BY publication_id
+  ) sp
+  ON sp.publication_id = publication.id
+LEFT OUTER JOIN
+  (
+  SELECT GROUP_CONCAT(DISTINCT prot, ', ') AS studies_with_data, publication_id
+  FROM (
+    studypublication
+    LEFT OUTER JOIN
+      (
+        SELECT study_name, data_availability FROM study
+      ) sdy
+      ON sdy.study_name = prot
+    )
+  WHERE data_availability IS NOT NULL
+  GROUP BY publication_id
+  ) da
+  ON da.publication_id = publication.id
+LEFT OUTER JOIN
+  (
+    SELECT filename, document_id, publication_id from learn_publicationdata
+  ) pd
+  ON pd.publication_id = publication.id
+"
+
+      availablePublications <- labkey.executeSql(
+        baseUrl = private$.config$labkeyUrlBase,
+        folderPath = "/CAVD/",
+        schemaName = "CDS",
+        sql = sqlQuery,
+        colNameOpt = "fieldname"
+      )
+
+      setDT(availablePublications)
+      setorder(availablePublications, "first_author")
+      setnames(availablePublications, "remote_path", "remotePath")
+      private$.availablePublications <- availablePublications
     }
   )
 )
